@@ -38,7 +38,6 @@
 #include <dlfcn.h>
 #include <glib.h>
 #include <glib/gprintf.h>
-#include <glib/gi18n.h>
 
 #include <dbus/dbus-glib-lowlevel.h>
 
@@ -60,7 +59,6 @@
 #include "mcd-misc.h"
 #include "plugin-loader.h"
 
-#include "libmcclient/mc-gtypes.h"
 #include "_gen/svc-dispatcher.h"
 
 #include <telepathy-glib/defs.h>
@@ -73,8 +71,6 @@
 #include <telepathy-glib/svc-generic.h>
 #include <telepathy-glib/util.h>
 
-#include <libmcclient/mc-errors.h>
-
 #include <stdlib.h>
 #include <string.h>
 #include "sp_timestamp.h"
@@ -412,7 +408,6 @@ _mcd_dispatcher_enter_state_machine (McdDispatcher *dispatcher,
     g_return_if_fail (channels != NULL);
     g_return_if_fail (MCD_IS_CHANNEL (channels->data));
     g_return_if_fail (requested || !only_observe);
-    g_return_if_fail (possible_handlers != NULL || only_observe);
 
     account = mcd_channel_get_account (channels->data);
     if (G_UNLIKELY (!account))
@@ -657,6 +652,90 @@ mcd_dispatcher_client_gone_cb (McdClientProxy *client,
     mcd_dispatcher_discard_client (self, client);
 }
 
+/*
+ * @channel: a non-null TpChannel which has already been dispatched
+ * @request: (allow-none): if not NULL, a request that resulted in
+ *     @channel
+ *
+ * Return a Handler to which @channel could be re-dispatched,
+ * for instance as a result of a re-request or a PresentChannel call.
+ *
+ * If @channel was dispatched to a Handler, return that Handler.
+ * Otherwise, if it was Claimed by a process, and that process
+ * has a Handler to which the channel could have been dispatched,
+ * return that Handler. Otherwise return NULL.
+ */
+static McdClientProxy *
+_mcd_dispatcher_lookup_handler (McdDispatcher *self,
+                                TpChannel *channel,
+                                McdRequest *request)
+{
+    McdClientProxy *handler = NULL;
+    const gchar *object_path;
+    const gchar *unique_name;
+    const gchar *well_known_name;
+
+    object_path = tp_proxy_get_object_path (channel);
+
+    unique_name = _mcd_handler_map_get_handler (self->priv->handler_map,
+                                                object_path,
+                                                &well_known_name);
+
+    if (unique_name == NULL)
+    {
+        DEBUG ("No process is handling channel %s", object_path);
+        return NULL;
+    }
+
+    if (well_known_name != NULL)
+    {
+        /* We know which Handler well-known name was responsible: use it if it
+         * still exists */
+        DEBUG ("Channel %s is handler by %s", object_path, well_known_name);
+        handler = _mcd_client_registry_lookup (self->priv->clients,
+                                               well_known_name);
+    }
+
+    if (handler == NULL)
+    {
+        GList *possible_handlers;
+        GList *channels;
+
+        /* Failing that, maybe the Handler it was dispatched to was temporary;
+         * try to pick another Handler that can deal with it, on the same
+         * unique name (i.e. in the same process).
+         * It can also happen in the case an Observer/Approver Claimed the
+         * channel; in that case we did not get its handler well known name.
+         */
+        channels = g_list_prepend (NULL, channel);
+        possible_handlers = _mcd_client_registry_list_possible_handlers (
+                self->priv->clients,
+                request != NULL ? _mcd_request_get_preferred_handler (request) : NULL,
+                request != NULL ? _mcd_request_get_properties (request) : NULL,
+                channels, unique_name);
+
+        if (possible_handlers != NULL)
+        {
+            DEBUG ("Pick first possible handler for channel %s", object_path);
+            handler = possible_handlers->data;
+        }
+        else
+        {
+            /* The process is still running (otherwise it wouldn't be in the
+             * handler map), but none of its well-known names is still
+             * interested in channels of that sort. Oh well, not our problem.
+             */
+            DEBUG ("process %s no longer interested in channel %s",
+                   unique_name, object_path);
+        }
+
+        g_list_free (channels);
+        g_list_free (possible_handlers);
+    }
+
+    return handler;
+}
+
 static void
 mcd_dispatcher_client_needs_recovery_cb (McdClientProxy *client,
                                          McdDispatcher *self)
@@ -675,19 +754,14 @@ mcd_dispatcher_client_needs_recovery_cb (McdClientProxy *client,
         TpChannel *channel = list->data;
         const gchar *object_path = tp_proxy_get_object_path (channel);
         GHashTable *properties;
-        const gchar *hname;
+        McdClientProxy *handler;
 
-        if (_mcd_handler_map_get_handler (self->priv->handler_map,
-              object_path, &hname))
+        /* FIXME: This is not exactly the right behaviour, see fd.o#40305 */
+        handler = _mcd_dispatcher_lookup_handler (self, channel, NULL);
+        if (handler && _mcd_client_proxy_get_bypass_observers (handler))
         {
-            McdClientProxy *handler =
-                _mcd_client_registry_lookup (self->priv->clients, hname);
-
-            if (_mcd_client_proxy_get_bypass_observers (handler))
-            {
-              DEBUG ("skipping unobservable channel %s", object_path);
-              continue;
-            }
+            DEBUG ("skipping unobservable channel %s", object_path);
+            continue;
         }
 
         properties = tp_channel_borrow_immutable_properties (channel);
@@ -701,7 +775,6 @@ mcd_dispatcher_client_needs_recovery_cb (McdClientProxy *client,
 
             _mcd_client_recover_observer (client, channel, account_path);
         }
-
     }
 
     /* we also need to think about channels that are still being dispatched,
@@ -775,7 +848,7 @@ mcd_dispatcher_client_registry_ready_cb (McdClientRegistry *clients,
     }
 
     g_ptr_array_foreach (vas, (GFunc) g_value_array_free, NULL);
-    g_ptr_array_free (vas, TRUE);
+    g_ptr_array_unref (vas);
 }
 
 static void
@@ -829,7 +902,7 @@ _mcd_dispatcher_dispose (GObject * object)
         tp_clear_object (&priv->clients);
     }
 
-    tp_clear_pointer (&priv->connections, g_hash_table_destroy);
+    tp_clear_pointer (&priv->connections, g_hash_table_unref);
     tp_clear_object (&priv->master);
     tp_clear_object (&priv->dbus_daemon);
 
@@ -866,7 +939,7 @@ mcd_dispatcher_update_client_caps (McdDispatcher *self,
     }
 
     g_ptr_array_foreach (vas, (GFunc) g_value_array_free, NULL);
-    g_ptr_array_free (vas, TRUE);
+    g_ptr_array_unref (vas);
 }
 
 static void
@@ -1432,9 +1505,8 @@ _mcd_dispatcher_take_channels (McdDispatcher *dispatcher, GList *channels,
     {
         if (channels->next == NULL)
         {
-            DEBUG ("One channel, which cannot be handled");
-            _mcd_channel_undispatchable (channels->data);
-            g_list_free (channels);
+            DEBUG ("One channel, which cannot be handled - making a CDO "
+                   "anyway, to get Observers run");
         }
         else
         {
@@ -1448,6 +1520,8 @@ _mcd_dispatcher_take_channels (McdDispatcher *dispatcher, GList *channels,
                 _mcd_dispatcher_take_channels (dispatcher, list, requested,
                                                FALSE);
             }
+
+            return;
         }
     }
     else
@@ -1455,16 +1529,16 @@ _mcd_dispatcher_take_channels (McdDispatcher *dispatcher, GList *channels,
         DEBUG ("%s handler(s) found, dispatching %u channels",
                internal_request ? "internal" : "possible",
                g_list_length (channels));
-
-        for (list = channels; list != NULL; list = list->next)
-            _mcd_channel_set_status (MCD_CHANNEL (list->data),
-                                     MCD_CHANNEL_STATUS_DISPATCHING);
-
-        _mcd_dispatcher_enter_state_machine (dispatcher, channels,
-            (const gchar * const *) possible_handlers, requested, FALSE);
-        g_list_free (channels);
     }
 
+    for (list = channels; list != NULL; list = list->next)
+        _mcd_channel_set_status (MCD_CHANNEL (list->data),
+                                 MCD_CHANNEL_STATUS_DISPATCHING);
+
+    _mcd_dispatcher_enter_state_machine (dispatcher, channels,
+        (const gchar * const *) possible_handlers, requested, FALSE);
+    g_list_free (channels);
+
     g_strfreev (possible_handlers);
 }
 
@@ -1561,18 +1635,14 @@ _mcd_dispatcher_reinvoke_handler (McdDispatcher *dispatcher,
                                   McdChannel *request)
 {
     GList *request_as_list;
-    const gchar *handler_unique;
-    const gchar *well_known_name = NULL;
-    GStrv possible_handlers = NULL;
     McdClientProxy *handler = NULL;
     McdRequest *real_request = _mcd_channel_get_request (request);
-    GList *tp_channels = g_list_append (NULL,
-        mcd_channel_get_tp_channel (request));
+    TpChannel *tp_channel = mcd_channel_get_tp_channel (request);
     GHashTable *handler_info;
     GHashTable *request_properties;
 
     g_assert (real_request != NULL);
-    g_assert (tp_channels->data != NULL);
+    g_assert (tp_channel != NULL);
 
     request_as_list = g_list_append (NULL, request);
 
@@ -1584,55 +1654,17 @@ _mcd_dispatcher_reinvoke_handler (McdDispatcher *dispatcher,
 
     handler_info = tp_asv_new (NULL, NULL);
     /* hand over ownership of request_properties */
-    /* FIXME: use telepathy-glib version when available */
     tp_asv_take_boxed (handler_info, "request-properties",
-                       MC_HASH_TYPE_OBJECT_IMMUTABLE_PROPERTIES_MAP,
+                       TP_HASH_TYPE_OBJECT_IMMUTABLE_PROPERTIES_MAP,
                        request_properties);
     request_properties = NULL;
 
-    /* the unique name (process) of the current handler */
-    handler_unique = _mcd_handler_map_get_handler (
-        dispatcher->priv->handler_map,
-        tp_proxy_get_object_path (tp_channels->data), &well_known_name);
-
-    if (well_known_name != NULL)
-    {
-        /* We know which Handler well-known name was responsible: if it
-         * still exists, we want to call HandleChannels on it */
-        handler = _mcd_client_registry_lookup (dispatcher->priv->clients,
-                                               well_known_name);
-    }
-
+    handler = _mcd_dispatcher_lookup_handler (dispatcher,
+            tp_channel, real_request);
     if (handler == NULL)
     {
-        /* Failing that, maybe the Handler it was dispatched to was temporary;
-         * try to pick another Handler that can deal with it, on the same
-         * unique name (i.e. in the same process) */
-        possible_handlers = mcd_dispatcher_dup_possible_handlers (dispatcher,
-            real_request, tp_channels, handler_unique);
-
-        if (possible_handlers == NULL || possible_handlers[0] == NULL)
-        {
-            /* The process is still running (otherwise it wouldn't be in the
-             * handler map), but none of its well-known names is still
-             * interested in channels of that sort. Oh well, not our problem.
-             */
-            DEBUG ("process %s no longer interested in this channel, not "
-                   "reinvoking", handler_unique);
-            mcd_dispatcher_finish_reinvocation (request);
-            goto finally;
-        }
-
-        handler = _mcd_client_registry_lookup (dispatcher->priv->clients,
-                                               possible_handlers[0]);
-
-        if (handler == NULL)
-        {
-            DEBUG ("Handler %s does not exist in client registry, not "
-                   "reinvoking", possible_handlers[0]);
-            mcd_dispatcher_finish_reinvocation (request);
-            goto finally;
-        }
+        mcd_dispatcher_finish_reinvocation (request);
+        goto finally;
     }
 
     /* This is deliberately not the same call as for normal dispatching,
@@ -1649,8 +1681,6 @@ _mcd_dispatcher_reinvoke_handler (McdDispatcher *dispatcher,
 finally:
     g_hash_table_unref (handler_info);
     g_list_free (request_as_list);
-    g_list_free (tp_channels);
-    g_strfreev (possible_handlers);
 }
 
 static McdDispatchOperation *
@@ -2095,7 +2125,7 @@ _mcd_dispatcher_add_connection (McdDispatcher *self,
         _mcd_connection_start_dispatching (connection, vas);
 
         g_ptr_array_foreach (vas, (GFunc) g_value_array_free, NULL);
-        g_ptr_array_free (vas, TRUE);
+        g_ptr_array_unref (vas);
     }
     /* else _mcd_connection_start_dispatching will be called when we're ready
      * for it */
@@ -2809,7 +2839,6 @@ dispatcher_present_channel (
     McdAccount *account;
     McdConnection *conn;
     McdChannel *mcd_channel;
-    const gchar *handler = NULL;
     GError *error = NULL;
     McdClientProxy *client;
     GList *channels = NULL;
@@ -2834,21 +2863,23 @@ dispatcher_present_channel (
     conn = mcd_account_get_connection (account);
     g_return_if_fail (conn != NULL);
 
-    _mcd_handler_map_get_handler (self->priv->handler_map, channel_path,
-        &handler);
-    if (handler == NULL)
+    mcd_channel = mcd_connection_find_channel_by_path (conn, channel_path);
+    g_return_if_fail (mcd_channel != NULL);
+
+    /* We take mcd_channel's request to base the search for a suitable Handler
+     * on the handler that was preferred by the request that initially created
+     * the Channel, if any.
+     * Actually not, because of fd.o#41031 */
+    client = _mcd_dispatcher_lookup_handler (self,
+            mcd_channel_get_tp_channel (mcd_channel),
+            _mcd_channel_get_request (mcd_channel));
+    if (client == NULL)
       {
         g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
             "Channel %s is currently not handled", channel_path);
         goto error;
       }
 
-    client = _mcd_client_registry_lookup (self->priv->clients, handler);
-    g_return_if_fail (client != NULL);
-
-    mcd_channel = mcd_connection_find_channel_by_path (conn, channel_path);
-    g_return_if_fail (mcd_channel != NULL);
-
     channels = g_list_append (channels, mcd_channel);
 
     _mcd_client_proxy_handle_channels (client, -1, channels,
