@@ -33,7 +33,6 @@
 #include <unistd.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/util.h>
-#include <libmcclient/mc-errors.h>
 
 #include "mcd-debug.h"
 
@@ -72,11 +71,6 @@ _mcd_build_error_string (const GError *error)
         klass = g_type_class_ref (TP_TYPE_ERROR);
         prefix = TP_ERROR_PREFIX;
     }
-    else if (error->domain == MC_ERROR)
-    {
-        klass = g_type_class_ref (MC_TYPE_ERROR);
-        prefix = MC_ERROR_PREFIX;
-    }
     else
         return NULL;
     value = g_enum_get_value (klass, error->code);
@@ -225,7 +219,7 @@ _mcd_chmod_private (const gchar *filename)
 
     if (ret < 0)
     {
-        DEBUG ("g_stat: %s", g_strerror (errno));
+        DEBUG ("g_stat(%s): %s", filename, g_strerror (errno));
         return ret;
     }
 
@@ -242,17 +236,3 @@ _mcd_chmod_private (const gchar *filename)
 
     return ret;
 }
-
-/* FIXME: fd.o#24876: remove this when we depend on GLib 2.22 */
-GError *
-_mcd_g_error_new_valist (GQuark domain,
-                         gint code,
-                         const gchar *format,
-                         va_list ap)
-{
-    gchar *message = g_strdup_vprintf (format, ap);
-    GError *error = g_error_new_literal (domain, code, message);
-
-    g_free (message);
-    return error;
-}
